﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Controllers;
using RESTWebAPI.Models;


namespace RESTWebAPI.Controllers
{
    public class ProductsController : ApiController
    {

        #region Members
        private readonly ProductRepository repository;
        #endregion Members

        #region Constructors
        public ProductsController()
            : this(new ProductRepository())
        {
        }

        public ProductsController(ProductRepository repository)
        {
            this.repository = repository;
        }
        #endregion

        #region GET
        // GET /api/products
        public IQueryable<Product> Get()
        {
            return this.repository.Products.AsQueryable<Product>();
        }

        // GET /api/products/id
        public Product Get(int id)
        {
            Product product = this.repository.Products.Find(p => p.ID == id);
            if (product == null)
                throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.NotFound));
            return product;
        }
        #endregion GET

        #region POST
        // POST /api/products
        // public HttpResponseMessage Post(Product product) is a duplicate action and 
        // ApiControllerActionSelector.SelectAction throws an exception
        public HttpResponseMessage Post(IEnumerable<Product> products)
        {
            if (ModelState.IsValid)
            {
                var product = this.repository.Products.FindAll(
                    p => products.Where<Product>(o => o.ID == p.ID).Count<Product>() > 0);
                if (product.Count() > 0)
                    throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.Conflict));

                this.repository.Create(products.ToList<Product>());

                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, products);
                // 201 Created should also return the location of the new resource.
                // As we are creating more than one we'll only add the location of the first.
                string uri = Url.Route(null, new { id = products.First<Product>().ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region PUT
        // PUT /api/products/5
        public HttpResponseMessage Put(int id, Product pr)
        {
            if (ModelState.IsValid)
            {
                if (this.repository.Update(id, pr))
                    /* Alternatively the response can be
                     * 200 OK if update was enacted correctly (same as 204 No Content)
                     * The response does not have to include a representation of the entity updated
                    return new HttpResponseMessage(HttpStatusCode.OK);
                     */ 
                    return new HttpResponseMessage(HttpStatusCode.NoContent);
                
                // if resource is not found it should be created
                this.repository.Create(pr);

                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, pr);
                // 201 Created should also return the location of the new resource.
                string uri = Url.Route(null, new { id = pr.ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }

        // PUT /api/products/?ID=5&ID=6&ID=7
        public HttpResponseMessage Put([FromUri]IEnumerable<int> index, IEnumerable<Product> products)
        {
            if (ModelState.IsValid)
            {
                List<Product> notFound = this.repository.Update(index.ToList<int>(), products.ToList<Product>());
                if (notFound.Count == 0)
                    /* Alternatively the response can be
                     * 200 OK if update was enacted correctly (same as 204 No Content)
                     * The response does not have to include a representation of the entity updated
                    return new HttpResponseMessage(HttpStatusCode.OK);
                     */ 
                    return new HttpResponseMessage(HttpStatusCode.NoContent);
                
                // resources which are not found should be created
                this.repository.Create(notFound);

                // create response
                HttpResponseMessage response = Request.CreateResponse(HttpStatusCode.Created, notFound);
                // 201 Created should also return the location of the new resource.
                // As we are creating more than one we'll only add the location of the first.
                string uri = Url.Route(null, new { id = notFound.First<Product>().ID });
                response.Headers.Location = new Uri(Request.RequestUri, uri);
                return response;

            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion

        #region DELETE
        // DELETE /api/products/5
        public HttpResponseMessage Delete(int id)
        {
            if (ModelState.IsValid)
            {
                return Request.CreateResponse(HttpStatusCode.OK, this.repository.Delete(id));
                /* Alternatively Delete can also return 
                 * 202 Accepted if the delete action is accepted but not enacted
                 * 204 No Content if the delete action has been enacted but the response does not include the deleted entity(s)
                return Request.CreateResponse(HttpStatusCode.NoContent);
                return Request.CreateResponse(HttpStatusCode.Accepted);
                */
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }

        // DELETE /api/products/?id=5&id=6&id=7
        // FromUri attribute is mandatory for MVC4 RC because the parameter is a complex type
        // and by default is taken from the request body
        public HttpResponseMessage Delete([FromUri]IEnumerable<int> index)
        {
            if (ModelState.IsValid)
            {
                return Request.CreateResponse(HttpStatusCode.OK, this.repository.Delete(index.ToList<int>()));
                /* Alternatively Delete can also return 
                 * 202 Accepted if the delete action is accepted but not enacted
                 * 204 No Content if the delete action has been enacted but the response does not include the deleted entity(s)
                return Request.CreateResponse(HttpStatusCode.NoContent);
                return Request.CreateResponse(HttpStatusCode.Accepted);
                */
            }
            throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.BadRequest));
        }
        #endregion
    }
}